# Linear regression model using gradient descent
# generate mock data by simulation

n_features=2
n_data=100
set.seed(11)
x<-matrix(rnorm(n_data*n_features),ncol=n_features)
#y<-rnorm(n_data)
X<-cbind(rep(1,n_data),x)  # the column 1 is added for the bias
theta_true=sample.int(10,n_features+1)  # generate the true model
y=X%*%theta_true    # generate the responses
theta_init<-rep(0,n_features+1)   # initialize theta

#  the loss function to be minimized

Loss<-function(X,y,theta){
  N=length(y)   # this should be equal to the number of data
  L=sum((X%*%theta-y)^2)/(2*N)
  return(L)
}


#  gradient descent

gradient_descent<-function(X,y,theta,alpha,max_iter){
  N=length(y)
  L_history<-rep(0,max_iter)
  for(i in 1:max_iter){
    theta<-theta-alpha*(1/N)*(t(X)%*%(X%*%theta-y))  # here we calculate
                                                     # the gradient explicitly
    L_history[i]<-Loss(X,y,theta)
  }
  results<-list(theta,L_history)
  return(results)
}


#  now execute 

alpha<-0.1
max_iter<-100
results<-gradient_descent(X,y,theta_init,alpha,max_iter)
theta<-results[[1]]   # retrieve theta vector
L_history<-results[[2]]  # retrieve L_history

# compare theta with theta_true

theta_true

theta

# plot Loss function as a function of the iteration number

plot(L_history)
